@@ -5,7 +5,7 @@ use warnings;
 package warnings::MaybeFatal;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.004';
+our $VERSION   = '0.005';
 
 BEGIN {
 	if ( $] < 5.012 ) {
@@ -17,6 +17,11 @@ BEGIN {
 use B::Hooks::EndOfScope;
 use Carp qw(croak);
 
+BEGIN {
+	B::Hooks::EndOfScope->Module::Implementation::implementation_for eq 'XS'
+		or croak("Pure Perl implementation of B::Hooks::EndOfScope not supported");
+};
+
 sub _my_hints
 {
 	$^H |= 0x20000;
@@ -29,14 +34,22 @@ sub import
 	
 	# Keep original signal handler
 	my $orig = $SIG{__WARN__};
-	my @warnings;
+	if (!ref($orig))
+	{
+		$orig = ($orig eq 'DEFAULT' or $orig eq 'IGNORE')
+			? undef
+			: do {
+				no strict 'refs';
+				exists(&{'main::'.$orig}) ? \&{'main::'.$orig} : undef;
+			};
+		$orig = sub { warn(@_) } unless defined $orig;
+	}
 	
+	my @warnings;
 	$SIG{__WARN__} = sub {
 		_my_hints->{+__PACKAGE__}
 			? push(@warnings, $_[0])
-			: $orig
-				? $orig->(@_)
-				: warn(@_);
+			: $orig->(@_);
 	};
 	
 	on_scope_end {
@@ -47,7 +60,7 @@ sub import
 		}
 		elsif (@warnings)
 		{
-			warn($_) for @warnings;
+			$orig->($_) for @warnings;
 			local $Carp::CarpLevel = $Carp::CarpLevel + 1;
 			croak("Compile time warnings");
 		}
@@ -118,7 +131,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2014 by Toby Inkster.
+This software is copyright (c) 2014, 2017 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
